package cmdToken

import (
    "errors"
    "encoding/json"
    u "nexus-client/utils"
    g "nexus-client/globals"
)

type Refresh struct {
    Name string
}

func (cmd *Refresh)GetName() string {
    return cmd.Name
}

func (cmd *Refresh)GetDesc() []string {
    return []string{
        "Obtains a new access token."}
}

func (cmd *Refresh)PrintUsage() {
    for _, desc := range cmd.GetDesc() {
        u.PrintlnErr(desc)
    }
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("USAGE: "+cmd.Name)
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
}

func (cmd *Refresh)Run(args []string) int {
    if len(args) > 0 {
        cmd.PrintUsage()
        return 1
    }

    token, err := GetToken()
    if err != nil {
        u.PrintlnErr("Error: "+err.Error())
        return 1
    }

    u.Println(token)
    return 0
}

func GetToken() (string, error) {
    client := g.GetInstance().Client
    host := g.GetInstance().Host

    resp, err := client.R().Get(host+"/token/refresh")
    if err != nil {
        return "", err
    } else {
        if resp.IsSuccess() {
            type Response struct {
                Token string
            }
            var response Response
            err = json.Unmarshal(resp.Body(), &response)
            if err != nil {
                return "", err
            }
            return response.Token, nil
        } else {
            return "", errors.New(resp.Status()+": "+resp.String())
        }
    }
}
