package exec

import (
    "fmt"
    "os/exec"
    "strings"
    "errors"
)

// Checks guestfish is available.
func CheckGuestfish() error {
    output, err := exec.Command(conf.Tools.Guestfish, "--version").Output()
    if err != nil {
        return errors.New(conf.Tools.Guestfish+" is required but not found. On Ubuntu/Debian, it can be installed with \"sudo apt-get install guestfish\".")
    }
    out := string(output)
    lines := strings.Split(out, "\n")
    fields := strings.Split(lines[0], " ")
    if len(fields) < 2 {
        return errors.New("Failed extracting "+conf.Tools.Guestfish+" version number!")
    }
    return nil
}

// Copies and unarchives a local tar.gz archive into a directory (vmDir) inside the VM's filesystem.
func CopyToVM(vmDiskFile, tarGzFile, vmDir string) error {
    cmd := exec.Command(conf.Tools.Guestfish, "--rw", "-i", "tar-in", "-a", vmDiskFile, tarGzFile, vmDir, "compress:gzip")
    stdoutStderr, err := cmd.CombinedOutput()
    if err != nil {
        output := fmt.Sprintf("[%s]", stdoutStderr)
        msg := "Failed writing to \""+vmDir+"\" in qcow ("+vmDiskFile+")"
        log.Error(msg+": "+output)
        return errors.New(msg)
    }
    return nil
}

// Recursively copies a directory in the VM's filesystem (vmDir) into a tar.gz archive.
func CopyFromVM(vmDiskFile, vmDir, tarGzFile string) error {
    cmd := exec.Command(conf.Tools.Guestfish, "--ro", "-i", "tar-out",  "-a", vmDiskFile, vmDir, tarGzFile, "compress:gzip")
    stdoutStderr, err := cmd.CombinedOutput()
    if err != nil {
        output := fmt.Sprintf("[%s]", stdoutStderr)
        msg := "Failed reading \""+vmDir+"\" in qcow ("+vmDiskFile+"): "+output
        log.Error(msg)
        return errors.New(msg)
    }
    return nil
}
